#include "gte_instruction.hpp"
#include <PSX/GTE/gte.hpp>

namespace JabyEngine {
    namespace GTE {
        void rot_trans(const SVECTOR& input, VECTOR& output, int32_t& flag) {
            ldv0(input);
            rt();
            stlvnl(output);
            stflg(flag);
        }

        void set_rot_matrix(const MATRIX& matrix) {
            __asm__ volatile("lw   $12, 0(%0)"  :: "r"(&matrix) : "$12", "$13", "$14"); 
            __asm__ volatile("lw   $13, 4(%0)"  :: "r"(&matrix) : "$12", "$13", "$14"); 
            __asm__ volatile("ctc2 $12, $0"     :: "r"(&matrix) : "$12", "$13", "$14"); 
            __asm__ volatile("ctc2 $13, $1"     :: "r"(&matrix) : "$12", "$13", "$14"); 
            __asm__ volatile("lw   $12, 8(%0)"  :: "r"(&matrix) : "$12", "$13", "$14"); 
            __asm__ volatile("lw   $13, 12(%0)" :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("lw   $14, 16(%0)" :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("ctc2 $12, $2"     :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("ctc2 $13, $3"     :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("ctc2 $14, $4"     :: "r"(&matrix) : "$12", "$13", "$14");
        }

        void set_trans_matrix(const MATRIX& matrix) {
            __asm__ volatile("lw   $12, 20(%0)" :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("lw   $13, 24(%0)" :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("ctc2 $12, $5"     :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("lw   $14, 28(%0)" :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("ctc2 $13, $6"     :: "r"(&matrix) : "$12", "$13", "$14");
            __asm__ volatile("ctc2 $14, $7"     :: "r"(&matrix) : "$12", "$13", "$14");
        }

        MATRIX& mult_matrix(const MATRIX& m0, const MATRIX& m1, MATRIX& result) {
            /*
                Jaby: Somehow this code creates stack usage.... Investigate!!
            */
            asm("# MY PLANSCHI START");   
            set_rot_matrix(m0);
            
            ldclmv(m1, 0);
            rtir();
            stclmv(result, 0);
            
            ldclmv(m1, 1);
            rtir();
            stclmv(result, 1);
            
            ldclmv(m1, 2);
            rtir();
            stclmv(result, 2);

            return result;
            asm("# MY PLANSCHI END");
        }

        void set_geom_offset(int32_t off_x, int32_t off_y) {
            __asm__ volatile("sll  $12, %0, 16" :: "r"(off_x), "r"(off_y) : "$12", "$13");
            __asm__ volatile("sll  $13, %1, 16" :: "r"(off_x), "r"(off_y) : "$12", "$13");
            __asm__ volatile("ctc2 $12, $24"    :: "r"(off_x), "r"(off_y) : "$12", "$13");
            __asm__ volatile("ctc2 $13, $25"    :: "r"(off_x), "r"(off_y) : "$12", "$13");
        }

        void set_geom_screen(int32_t h) {
            __asm__ volatile("ctc2 %0, $26" :: "r"(h));
        }
    }
}